@@ -6,19 +6,20 @@ import threading
 
 
 class DepthAIPipeline:
-    def __init__(self, nn_path, size, resolution, class_names, cam_stream, colors, confidence=0.5, overlap=0.5, sensor_mode="THE_1080_P", stretch=False, depth=False, device=None, legacy=False):
+    def __init__(self, nn_path, size, resolution, class_names, cam_stream, colors, fast=True, confidence=0.5, overlap=0.5, sensor_mode="THE_1080_P", stretch=False, depth=False, device=None, legacy=False):
         self.nn_path = nn_path
         self.size = size
         self.class_names = class_names
         self.cam_stream = cam_stream
         self.colors = colors
+        self.fast = fast
         self.stretch = stretch
         self.resolution = resolution
         self.dev = device
         self.depth = depth
         self.overlap = overlap
         self.confidence = confidence
-        self.frames = []
+        self.frames = {}
 
         self.pipeline = dai.Pipeline()
         if legacy:
@@ -26,7 +27,6 @@ class DepthAIPipeline:
 
         self.cam_rgb = self.pipeline.create(dai.node.ColorCamera)
         self.cam_rgb.setBoardSocket(dai.CameraBoardSocket.RGB)
-        self.cam_rgb.setResolution(getattr(dai.ColorCameraProperties.SensorResolution, sensor_mode))
         self.cam_rgb.setColorOrder(dai.ColorCameraProperties.ColorOrder.RGB)
         self.cam_rgb.setInterleaved(False)
 
@@ -55,26 +55,38 @@ class DepthAIPipeline:
         self.xout_nn.setStreamName("nn")
         self.detection_nn.out.link(self.xout_nn.input)
 
-
-        self.manip = self.pipeline.createImageManip()
-
-        self.manip.initialConfig.setCropRect((0,0,1,1))
-        self.manip.initialConfig.setResize(self.size)
-
         self.manip_out = self.pipeline.createXLinkOut()
         self.manip_out.setStreamName("manip")
-        self.manip.out.link(self.manip_out.input)
 
-        if self.stretch:
-            self.manip.initialConfig.setKeepAspectRatio(False)
-        self.manip.inputImage.setBlocking(True)
-        self.manip.initialConfig.setFrameType(dai.ImgFrame.Type.BGR888p)
 
-        self.manip.out.link(self.detection_nn.input)
-        self.cam_rgb.video.link(self.manip.inputImage)
-        if cam_stream:
-            self.cam_rgb.video.link(self.xout_raw.input)
-            self.manip.out.link(self.xout_rgb.input)
+        if self.fast:
+            #self.cam_rgb.setResolution(getattr(dai.ColorCameraProperties.SensorResolution, "THE_720_P"))
+            self.cam_rgb.setPreviewSize(self.size)
+            self.cam_rgb.preview.link(self.detection_nn.input)
+            if cam_stream:
+                self.detection_nn.passthrough.link(self.xout_raw.input)
+
+        else:
+            self.cam_rgb.setResolution(getattr(dai.ColorCameraProperties.SensorResolution, sensor_mode))
+
+            self.manip = self.pipeline.createImageManip()
+
+            self.manip.initialConfig.setCropRect((0,0,1,1))
+            self.manip.initialConfig.setResize(self.size)
+            self.manip.setMaxOutputFrameSize(self.size[0]*self.size[1]*3)
+            self.manip.out.link(self.manip_out.input)
+
+            if self.stretch:
+                self.manip.initialConfig.setKeepAspectRatio(False)
+            #self.manip.inputImage.setBlocking(True)
+            self.manip.initialConfig.setFrameType(dai.ImgFrame.Type.BGR888p)
+
+            self.manip.out.link(self.detection_nn.input)
+
+            self.cam_rgb.video.link(self.manip.inputImage)
+            if cam_stream:
+                self.cam_rgb.video.link(self.xout_raw.input)
+                self.manip.out.link(self.xout_rgb.input)
 
         if self.depth:
             self.left = self.pipeline.create(dai.node.MonoCamera)
@@ -99,7 +111,7 @@ class DepthAIPipeline:
 
             self.left.out.link(self.stereo.left)
             self.right.out.link(self.stereo.right)
-            self.stereo.disparity.link(self.depthOut.input)
+            self.stereo.depth.link(self.depthOut.input)
 
 
         if self.dev is None:
@@ -127,13 +139,11 @@ class DepthAIPipeline:
 
         self.q_manip = self.device.getOutputQueue(name="manip", maxSize=4, blocking=False)
 
-        frame_thread = threading.Thread(target=collect_frame_thread, args=(self,))
-        frame_thread.start()
-
-    def disparity_to_depth(self, disparity):
-        if disparity == 0:
-            return np.NaN
-        return 441.25 * 7.5 / disparity
+        if not self.fast:
+            frame_thread = threading.Thread(target=collect_frame_thread, args=(self,))
+            frame_thread.start()
+            frame_thread2 = threading.Thread(target=collect_frame_thread, args=(self,))
+            frame_thread2.start()
 
     def detection_depth(self, detections, depth):
         res = []
@@ -142,14 +152,12 @@ class DepthAIPipeline:
 
         depth = np.nan_to_num(depth, copy=False, nan=0)
 
-        depth_map = self.disparity_to_depth(depth)
-
         for det in detections:
             x = (det[2]+det[0])/2
             y = (det[3]+det[1])/2
             x = int(x*sx)
             y = int(y*sy)
-            d = depth_map[det[0]:det[2], det[1]:det[3]]
+            d = depth[det[0]:det[2], det[1]:det[3]]
             dist = np.amin(d)
             res.append([det[0], det[1], det[2], det[3], det[4], det[5], dist])
 
@@ -159,15 +167,29 @@ class DepthAIPipeline:
         q_control = self.device.getInputQueue(name="control")
         q_control.send(control)
 
-
     def print_seq_nums(self):
         nums = [frame.getSequenceNum() for frame in self.frames]
         print("Current Frames: ", nums)
+
+    def clear_earlier_frames(self, seq_num):
+        self.frames = {key:val for key, val in self.frames.items() if int(key) >= seq_num}
+
     def find_frame(self, in_det):
-        for i, frame in enumerate(self.frames):
-            if frame.getSequenceNum() == in_det.getSequenceNum():
-                self.frames = self.frames[i:]
-                return frame
+        in_det_seq = in_det.getSequenceNum()
+        #print("in_det seq num", in_det_seq)
+        #self.print_seq_nums()
+        self.clear_earlier_frames(in_det_seq)
+        try:
+            return self.frames[str(in_det_seq)]
+        except:
+            return
+        #for i, frame in enumerate(self.frames):
+        #    if frame.getSequenceNum() == in_det_seq:
+        #        self.frames = self.frames[i:]
+        #        return frame
+
+        #if len(self.frames) > 0:
+        #    return self.frames[1]
 
 
 
@@ -175,33 +197,38 @@ class DepthAIPipeline:
         if self.depth:
             in_depth = self.q_depth.get()
 
-        in_manip = self.q_manip.get()
+        #in_manip = self.q_manip.get()
 
         in_det = self.q_det.get()
-
+        print("getting detections")
         detections = self.post_processing(in_det)
 
         depth = None
 
         if self.depth:
             depth = in_depth.getFrame()
+
             detections = self.detection_depth(detections, depth)
 
-
         if self.cam_stream:
-            frame, frame_raw = self.process_frame(detections, in_det)
+            rgb_in = None
+            if self.fast:
+                rgb_in = self.q_raw.get()
+            frame, frame_raw = self.process_frame(detections, in_det, rgb_in)
             return detections, frame, frame_raw, depth
 
         return detections, depth
 
-    def process_frame(self, detections, in_det):
+    def process_frame(self, detections, in_det, in_raw):
         frame_raw = None
-        frame = None
-        in_raw = self.find_frame(in_det)
+        if not self.fast:
+            in_raw = self.find_frame(in_det)
+
         if not in_raw:
             return np.array([]), np.array([])
 
         frame = in_raw.getCvFrame()
+
         x_offset = int((in_raw.getWidth()-in_raw.getHeight())/2)
         x_max = in_raw.getWidth()-x_offset
 
@@ -261,9 +288,12 @@ class DepthAIPipeline:
 def collect_frame_thread(self):
     while True:
         if self.cam_stream:
-            in_rgb = self.q_rgb.get()
+            #in_rgb = self.q_rgb.get()
             in_raw = self.q_raw.get()
-            self.frames.append(in_raw)
+            self.frames[str(in_raw.getSequenceNum())] = in_raw
+            #print("got frame; queue: ",len(self.frames))
+        else:
+            break
 def list_devices():
     available_devices = []
     for device in dai.Device.getAllAvailableDevices():
